@@ -1,6 +1,7 @@
 # SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 from .ormsgpack import (
+    OPT_DATETIME_AS_TIMESTAMP_EXT,
     OPT_NAIVE_UTC,
     OPT_NON_STR_KEYS,
     OPT_OMIT_MICROSECONDS,
@@ -11,6 +12,7 @@ from .ormsgpack import (
     OPT_PASSTHROUGH_SUBCLASS,
     OPT_PASSTHROUGH_TUPLE,
     OPT_PASSTHROUGH_UUID,
+    OPT_REPLACE_SURROGATES,
     OPT_SERIALIZE_NUMPY,
     OPT_SERIALIZE_PYDANTIC,
     OPT_SORT_KEYS,
@@ -30,6 +32,7 @@ __all__ = (
     "Ext",
     "MsgpackDecodeError",
     "MsgpackEncodeError",
+    "OPT_DATETIME_AS_TIMESTAMP_EXT",
     "OPT_NAIVE_UTC",
     "OPT_NON_STR_KEYS",
     "OPT_OMIT_MICROSECONDS",
@@ -40,6 +43,7 @@ __all__ = (
     "OPT_PASSTHROUGH_SUBCLASS",
     "OPT_PASSTHROUGH_TUPLE",
     "OPT_PASSTHROUGH_UUID",
+    "OPT_REPLACE_SURROGATES",
     "OPT_SERIALIZE_NUMPY",
     "OPT_SERIALIZE_PYDANTIC",
     "OPT_SORT_KEYS",
