@@ -1,14 +1,13 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-use crate::deserialize::cache::*;
 use crate::deserialize::DeserializeError;
 use crate::exc::*;
 use crate::ffi::*;
 use crate::msgpack::Marker;
 use crate::opt::*;
-use crate::typeref::*;
-use crate::unicode::*;
+use crate::state::State;
 use byteorder::{BigEndian, ReadBytesExt};
+use chrono::{Datelike, Timelike};
 use simdutf8::basic::{from_utf8, Utf8Error};
 use std::borrow::Cow;
 use std::os::raw::c_char;
@@ -18,36 +17,32 @@ const RECURSION_LIMIT: u16 = 1024;
 
 pub fn deserialize(
     ptr: *mut pyo3::ffi::PyObject,
+    state: *mut State,
     ext_hook: Option<NonNull<pyo3::ffi::PyObject>>,
     opts: Opt,
 ) -> Result<NonNull<pyo3::ffi::PyObject>, DeserializeError<'static>> {
     let obj_type_ptr = ob_type!(ptr);
-    let buffer: *const u8;
-    let length: usize;
+    let contents: &[u8];
 
-    if py_is!(obj_type_ptr, BYTES_TYPE) {
-        buffer = unsafe { PyBytes_AS_STRING(ptr) as *const u8 };
-        length = unsafe { PyBytes_GET_SIZE(ptr) as usize };
-    } else if py_is!(obj_type_ptr, MEMORYVIEW_TYPE) {
-        let membuf = unsafe { PyMemoryView_GET_BUFFER(ptr) };
-        if unsafe { pyo3::ffi::PyBuffer_IsContiguous(membuf, b'C' as c_char) == 0 } {
+    if obj_type_ptr == &raw mut pyo3::ffi::PyBytes_Type {
+        contents = unsafe { pybytes_as_bytes(ptr) };
+    } else if obj_type_ptr == &raw mut pyo3::ffi::PyMemoryView_Type {
+        if let Some(buffer) = unsafe { pymemoryview_as_bytes(ptr) } {
+            contents = buffer;
+        } else {
             return Err(DeserializeError::new(Cow::Borrowed(
                 "Input type memoryview must be a C contiguous buffer",
             )));
         }
-        buffer = unsafe { (*membuf).buf as *const u8 };
-        length = unsafe { (*membuf).len as usize };
-    } else if py_is!(obj_type_ptr, BYTEARRAY_TYPE) {
-        buffer = ffi!(PyByteArray_AsString(ptr)) as *const u8;
-        length = ffi!(PyByteArray_Size(ptr)) as usize;
+    } else if obj_type_ptr == &raw mut pyo3::ffi::PyByteArray_Type {
+        contents = unsafe { pybytearray_as_bytes(ptr) };
     } else {
         return Err(DeserializeError::new(Cow::Borrowed(
             "Input must be bytes, bytearray, memoryview",
         )));
     }
-    let contents: &[u8] = unsafe { std::slice::from_raw_parts(buffer, length) };
 
-    let mut deserializer = Deserializer::new(contents, ext_hook, opts);
+    let mut deserializer = Deserializer::new(contents, state, ext_hook, opts);
     deserializer
         .deserialize()
         .map_err(|e| DeserializeError::new(Cow::Owned(e.to_string())))
@@ -60,6 +55,7 @@ enum Error {
     Internal,
     InvalidStr,
     InvalidType(Marker),
+    InvalidValue,
     RecursionLimitReached,
     UnexpectedEof,
 }
@@ -71,10 +67,11 @@ impl std::fmt::Display for Error {
             Error::ExtHookFailed => f.write_str("ext_hook failed"),
             Error::ExtHookMissing => f.write_str("ext_hook missing"),
             Error::Internal => f.write_str("internal error"),
-            Error::InvalidStr => f.write_str(INVALID_STR),
+            Error::InvalidStr => f.write_str("invalid UTF-8 string"),
             Error::InvalidType(ref marker) => {
                 write!(f, "invalid type {marker:?}")
             }
+            Error::InvalidValue => f.write_str("invalid value"),
             Error::RecursionLimitReached => f.write_str(RECURSION_LIMIT_REACHED),
             Error::UnexpectedEof => write!(f, "unexpected end of file"),
         }
@@ -90,15 +87,22 @@ impl From<Utf8Error> for Error {
 
 struct Deserializer<'de> {
     data: &'de [u8],
+    state: *mut State,
     ext_hook: Option<NonNull<pyo3::ffi::PyObject>>,
     opts: Opt,
     recursion: u16,
 }
 
 impl<'de> Deserializer<'de> {
-    fn new(data: &'de [u8], ext_hook: Option<NonNull<pyo3::ffi::PyObject>>, opts: Opt) -> Self {
+    fn new(
+        data: &'de [u8],
+        state: *mut State,
+        ext_hook: Option<NonNull<pyo3::ffi::PyObject>>,
+        opts: Opt,
+    ) -> Self {
         Deserializer {
             data: data,
+            state: state,
             ext_hook: ext_hook,
             opts: opts,
             recursion: 0,
@@ -186,93 +190,157 @@ impl<'de> Deserializer<'de> {
         Ok(Marker::from_u8(n))
     }
 
+    fn deserialize_timestamp_ext(
+        &mut self,
+        len: u32,
+    ) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
+        let (seconds, nanoseconds): (i64, u32) = match len {
+            4 => {
+                let seconds = self.read_u32()?;
+                (seconds.into(), 0)
+            }
+            8 => {
+                let value = self.read_u64()?;
+                ((value & 0x3ffffffff) as i64, (value >> 34) as u32)
+            }
+            12 => {
+                let nanoseconds = self.read_u32()?;
+                let seconds = self.read_i64()?;
+                (seconds, nanoseconds)
+            }
+            _ => return Err(Error::InvalidValue),
+        };
+        let datetime = match chrono::DateTime::<chrono::Utc>::from_timestamp(seconds, nanoseconds) {
+            Some(value) => value,
+            None => return Err(Error::InvalidValue),
+        };
+        unsafe {
+            let obj = {
+                let datetime_api = *pyo3::ffi::PyDateTimeAPI();
+                (datetime_api.DateTime_FromDateAndTime)(
+                    datetime.year(),
+                    datetime.month() as i32,
+                    datetime.day() as i32,
+                    datetime.hour() as i32,
+                    datetime.minute() as i32,
+                    datetime.second() as i32,
+                    (datetime.nanosecond() / 1000) as i32,
+                    datetime_api.TimeZone_UTC,
+                    datetime_api.DateTimeType,
+                )
+            };
+            Ok(NonNull::new_unchecked(obj))
+        }
+    }
+
     fn deserialize_ext(&mut self, len: u32) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
         let tag = self.read_i8()?;
+        if tag == -1 && self.opts & DATETIME_AS_TIMESTAMP_EXT != 0 {
+            return self.deserialize_timestamp_ext(len);
+        }
+
         let data = self.read_slice(len as usize)?;
 
         match self.ext_hook {
-            Some(callable) => {
-                let tag_obj = ffi!(PyLong_FromLongLong(tag as i64));
-                let data_ptr = data.as_ptr() as *const c_char;
+            Some(callable) => unsafe {
+                let tag_obj = pyo3::ffi::PyLong_FromLongLong(tag as i64);
+                let data_ptr = data.as_ptr().cast::<c_char>();
                 let data_len = data.len() as pyo3::ffi::Py_ssize_t;
-                let data_obj = ffi!(PyBytes_FromStringAndSize(data_ptr, data_len));
-                #[allow(clippy::unnecessary_cast)]
-                let obj = ffi!(PyObject_CallFunctionObjArgs(
+                let data_obj = pyo3::ffi::PyBytes_FromStringAndSize(data_ptr, data_len);
+                let obj = pyo3::ffi::PyObject_CallFunctionObjArgs(
                     callable.as_ptr(),
                     tag_obj,
                     data_obj,
-                    std::ptr::null_mut() as *mut pyo3::ffi::PyObject
-                ));
-                ffi!(Py_DECREF(tag_obj));
-                ffi!(Py_DECREF(data_obj));
+                    std::ptr::null_mut::<pyo3::ffi::PyObject>(),
+                );
+                pyo3::ffi::Py_DECREF(tag_obj);
+                pyo3::ffi::Py_DECREF(data_obj);
                 if unlikely!(obj.is_null()) {
                     Err(Error::ExtHookFailed)
                 } else {
-                    Ok(nonnull!(obj))
+                    Ok(NonNull::new_unchecked(obj))
                 }
-            }
+            },
             None => Err(Error::ExtHookMissing),
         }
     }
 
     fn deserialize_null(&self) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        ffi!(Py_INCREF(NONE));
-        Ok(nonnull!(NONE))
+        unsafe {
+            let py_none = pyo3::ffi::Py_None();
+            pyo3::ffi::Py_INCREF(py_none);
+            Ok(NonNull::new_unchecked(py_none))
+        }
     }
 
     fn deserialize_true(&self) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        ffi!(Py_INCREF(TRUE));
-        Ok(nonnull!(TRUE))
+        unsafe {
+            let py_true = pyo3::ffi::Py_True();
+            pyo3::ffi::Py_INCREF(py_true);
+            Ok(NonNull::new_unchecked(py_true))
+        }
     }
 
     fn deserialize_false(&self) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        ffi!(Py_INCREF(FALSE));
-        Ok(nonnull!(FALSE))
+        unsafe {
+            let py_false = pyo3::ffi::Py_False();
+            pyo3::ffi::Py_INCREF(py_false);
+            Ok(NonNull::new_unchecked(py_false))
+        }
     }
 
     fn deserialize_i64(&self, value: i64) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        Ok(nonnull!(ffi!(PyLong_FromLongLong(value))))
+        unsafe {
+            let ptr = pyo3::ffi::PyLong_FromLongLong(value);
+            Ok(NonNull::new_unchecked(ptr))
+        }
     }
 
     fn deserialize_u64(&self, value: u64) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        Ok(nonnull!(ffi!(PyLong_FromUnsignedLongLong(value))))
+        unsafe {
+            let ptr = pyo3::ffi::PyLong_FromUnsignedLongLong(value);
+            Ok(NonNull::new_unchecked(ptr))
+        }
     }
 
     fn deserialize_f64(&self, value: f64) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        Ok(nonnull!(ffi!(PyFloat_FromDouble(value))))
+        unsafe {
+            let ptr = pyo3::ffi::PyFloat_FromDouble(value);
+            Ok(NonNull::new_unchecked(ptr))
+        }
     }
 
     fn deserialize_str(&mut self, len: u32) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
         let data = self.read_slice(len as usize)?;
         let value = from_utf8(data)?;
-        Ok(nonnull!(unicode_from_str(value)))
+        let ptr = unicode_from_str(value);
+        unsafe { Ok(NonNull::new_unchecked(ptr)) }
     }
 
     fn deserialize_bin(&mut self, len: u32) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
         let v = self.read_slice(len as usize)?;
-        let ptr = v.as_ptr() as *const c_char;
+        let ptr = v.as_ptr().cast::<c_char>();
         let len = v.len() as pyo3::ffi::Py_ssize_t;
-        Ok(nonnull!(ffi!(PyBytes_FromStringAndSize(ptr, len))))
+        unsafe {
+            let ptr = pyo3::ffi::PyBytes_FromStringAndSize(ptr, len);
+            Ok(NonNull::new_unchecked(ptr))
+        }
     }
 
     fn deserialize_array(&mut self, len: u32) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        let ptr = ffi!(PyList_New(len as pyo3::ffi::Py_ssize_t));
+        let ptr = unsafe { pyo3::ffi::PyList_New(len as pyo3::ffi::Py_ssize_t) };
         for i in 0..len {
             let elem = self.deserialize()?;
-            ffi!(PyList_SET_ITEM(
-                ptr,
-                i as pyo3::ffi::Py_ssize_t,
-                elem.as_ptr()
-            ));
+            unsafe { pyo3::ffi::PyList_SET_ITEM(ptr, i as pyo3::ffi::Py_ssize_t, elem.as_ptr()) };
         }
-        Ok(nonnull!(ptr))
+        unsafe { Ok(NonNull::new_unchecked(ptr)) }
     }
 
     fn deserialize_map_with_str_keys(
         &mut self,
         len: u32,
     ) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        let dict_ptr = ffi!(_PyDict_NewPresized(len as pyo3::ffi::Py_ssize_t));
+        let dict_ptr = unsafe { pydict_new_presized(len as pyo3::ffi::Py_ssize_t) };
         for _ in 0..len {
             let marker = self.read_marker()?;
             let key = match marker {
@@ -292,36 +360,35 @@ impl<'de> Deserializer<'de> {
                 marker => Err(Error::InvalidType(marker)),
             }?;
             let value = self.deserialize()?;
-            let pyhash = unsafe { (*key.as_ptr().cast::<pyo3::ffi::PyASCIIObject>()).hash };
-            let _ = ffi!(_PyDict_SetItem_KnownHash(
-                dict_ptr,
-                key.as_ptr(),
-                value.as_ptr(),
-                pyhash
-            ));
-            // counter Py_INCREF in insertdict
-            ffi!(Py_DECREF(key.as_ptr()));
-            ffi!(Py_DECREF(value.as_ptr()));
+            unsafe {
+                let pyhash = (*key.as_ptr().cast::<pyo3::ffi::PyASCIIObject>()).hash;
+                let _ = pydict_set_item_known_hash(dict_ptr, key.as_ptr(), value.as_ptr(), pyhash);
+                // counter Py_INCREF in insertdict
+                pyo3::ffi::Py_DECREF(key.as_ptr());
+                pyo3::ffi::Py_DECREF(value.as_ptr());
+            }
         }
-        Ok(nonnull!(dict_ptr))
+        unsafe { Ok(NonNull::new_unchecked(dict_ptr)) }
     }
 
     fn deserialize_map_with_non_str_keys(
         &mut self,
         len: u32,
     ) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        let dict_ptr = ffi!(_PyDict_NewPresized(len as pyo3::ffi::Py_ssize_t));
+        let dict_ptr = unsafe { pydict_new_presized(len as pyo3::ffi::Py_ssize_t) };
         for _ in 0..len {
             let key = self.deserialize_map_key()?;
             let value = self.deserialize()?;
-            let ret = ffi!(PyDict_SetItem(dict_ptr, key.as_ptr(), value.as_ptr()));
-            if unlikely!(ret == -1) {
-                return Err(Error::Internal);
+            unsafe {
+                let ret = pyo3::ffi::PyDict_SetItem(dict_ptr, key.as_ptr(), value.as_ptr());
+                pyo3::ffi::Py_DECREF(key.as_ptr());
+                pyo3::ffi::Py_DECREF(value.as_ptr());
+                if unlikely!(ret == -1) {
+                    return Err(Error::Internal);
+                }
             }
-            ffi!(Py_DECREF(key.as_ptr()));
-            ffi!(Py_DECREF(value.as_ptr()));
         }
-        Ok(nonnull!(dict_ptr))
+        unsafe { Ok(NonNull::new_unchecked(dict_ptr)) }
     }
 
     fn deserialize_map(&mut self, len: u32) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
@@ -459,8 +526,7 @@ impl<'de> Deserializer<'de> {
             Ok(value)
         } else {
             let data = self.read_slice(len as usize)?;
-            let map = unsafe { KEY_MAP.get_mut().unwrap_or_else(|| unreachable!()) };
-            Ok(map.get(data)?)
+            Ok(unsafe { (*self.state).key_map.get(data)? })
         }
     }
 
@@ -468,16 +534,23 @@ impl<'de> Deserializer<'de> {
         &mut self,
         len: u32,
     ) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
-        let ptr = ffi!(PyTuple_New(len as pyo3::ffi::Py_ssize_t));
+        let ptr = unsafe { pyo3::ffi::PyTuple_New(len as pyo3::ffi::Py_ssize_t) };
         for i in 0..len {
             let elem = self.deserialize_map_key()?;
-            ffi!(PyTuple_SET_ITEM(
-                ptr,
-                i as pyo3::ffi::Py_ssize_t,
-                elem.as_ptr()
-            ));
+            unsafe {
+                pytuple_set_item(ptr, i as pyo3::ffi::Py_ssize_t, elem.as_ptr());
+            }
+        }
+        unsafe { Ok(NonNull::new_unchecked(ptr)) }
+    }
+
+    fn deserialize_map_ext_key(&mut self, len: u32) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
+        let tag = self.read_i8()?;
+        if tag == -1 && self.opts & DATETIME_AS_TIMESTAMP_EXT != 0 {
+            self.deserialize_timestamp_ext(len)
+        } else {
+            Err(Error::InvalidValue)
         }
-        Ok(nonnull!(ptr))
     }
 
     fn deserialize_map_key(&mut self) -> Result<NonNull<pyo3::ffi::PyObject>, Error> {
@@ -567,6 +640,12 @@ impl<'de> Deserializer<'de> {
                 let len = self.read_u32()?;
                 self.deserialize_map_array_key(len)
             }
+            Marker::FixExt4 => self.deserialize_map_ext_key(4),
+            Marker::FixExt8 => self.deserialize_map_ext_key(8),
+            Marker::Ext8 => {
+                let len = self.read_u8()?;
+                self.deserialize_map_ext_key(len.into())
+            }
             marker => Err(Error::InvalidType(marker)),
         };
 
