@@ -16,6 +16,8 @@ pub const SORT_KEYS: Opt = 1 << 10;
 pub const UTC_Z: Opt = 1 << 11;
 pub const PASSTHROUGH_UUID: Opt = 1 << 12;
 pub const PASSTHROUGH_ENUM: Opt = 1 << 13;
+pub const DATETIME_AS_TIMESTAMP_EXT: Opt = 1 << 14;
+pub const REPLACE_SURROGATES: Opt = 1 << 15;
 
 pub const NOT_PASSTHROUGH: Opt = !(PASSTHROUGH_BIG_INT
     | PASSTHROUGH_DATACLASS
@@ -25,7 +27,8 @@ pub const NOT_PASSTHROUGH: Opt = !(PASSTHROUGH_BIG_INT
     | PASSTHROUGH_TUPLE
     | PASSTHROUGH_UUID);
 
-pub const PACKB_OPT_MASK: i32 = (NAIVE_UTC
+pub const PACKB_OPT_MASK: i32 = (DATETIME_AS_TIMESTAMP_EXT
+    | NAIVE_UTC
     | NON_STR_KEYS
     | OMIT_MICROSECONDS
     | PASSTHROUGH_BIG_INT
@@ -35,9 +38,10 @@ pub const PACKB_OPT_MASK: i32 = (NAIVE_UTC
     | PASSTHROUGH_SUBCLASS
     | PASSTHROUGH_TUPLE
     | PASSTHROUGH_UUID
+    | REPLACE_SURROGATES
     | SERIALIZE_NUMPY
     | SERIALIZE_PYDANTIC
     | SORT_KEYS
     | UTC_Z) as i32;
 
-pub const UNPACKB_OPT_MASK: i32 = NON_STR_KEYS as i32;
+pub const UNPACKB_OPT_MASK: i32 = (DATETIME_AS_TIMESTAMP_EXT | NON_STR_KEYS) as i32;
