@@ -1,17 +1,17 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 use crate::exc::*;
-use crate::ffi::PyDictIter;
+use crate::ffi::*;
 use crate::opt::*;
 use crate::serialize::serializer::*;
-use crate::typeref::*;
-use crate::unicode::*;
+use crate::state::State;
 use serde::ser::{Serialize, SerializeMap, Serializer};
 use smallvec::SmallVec;
 use std::ptr::NonNull;
 
 pub struct Dict {
     ptr: *mut pyo3::ffi::PyObject,
+    state: *mut State,
     opts: Opt,
     default_calls: u8,
     recursion: u8,
@@ -21,12 +21,14 @@ pub struct Dict {
 impl Dict {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
+        state: *mut State,
         opts: Opt,
         default_calls: u8,
         recursion: u8,
         default: Option<NonNull<pyo3::ffi::PyObject>>,
     ) -> Self {
         Dict {
+            state: state,
             ptr: ptr,
             opts: opts,
             default_calls: default_calls,
@@ -42,142 +44,66 @@ impl Serialize for Dict {
     where
         S: Serializer,
     {
-        if unlikely!(ffi!(Py_SIZE(self.ptr)) == 0) {
+        let mut critical_section = CriticalSection::new();
+        critical_section.begin(self.ptr);
+        if unlikely!(unsafe { pydict_size(self.ptr) } == 0) {
             serializer.serialize_map(Some(0)).unwrap().end()
         } else if self.opts & (NON_STR_KEYS | SORT_KEYS) == 0 {
-            DictWithStrKeys::new(
-                self.ptr,
-                self.opts,
-                self.default_calls,
-                self.recursion,
-                self.default,
-            )
-            .serialize(serializer)
+            self.serialize_with_str_keys(serializer)
         } else if self.opts & NON_STR_KEYS != 0 {
             if self.opts & SORT_KEYS != 0 {
-                err!("OPT_NON_STR_KEYS is not compatible with OPT_SORT_KEYS")
+                return Err(serde::ser::Error::custom(
+                    "OPT_NON_STR_KEYS is not compatible with OPT_SORT_KEYS",
+                ));
             }
-            DictWithNonStrKeys::new(
-                self.ptr,
-                self.opts,
-                self.default_calls,
-                self.recursion,
-                self.default,
-            )
-            .serialize(serializer)
+            self.serialize_with_non_str_keys(serializer)
         } else {
-            DictWithSortedStrKeys::new(
-                self.ptr,
-                self.opts,
-                self.default_calls,
-                self.recursion,
-                self.default,
-            )
-            .serialize(serializer)
+            self.serialize_with_sorted_str_keys(serializer)
         }
     }
 }
 
-pub struct DictWithStrKeys {
-    ptr: *mut pyo3::ffi::PyObject,
-    opts: Opt,
-    default_calls: u8,
-    recursion: u8,
-    default: Option<NonNull<pyo3::ffi::PyObject>>,
-}
-
-impl DictWithStrKeys {
-    pub fn new(
-        ptr: *mut pyo3::ffi::PyObject,
-        opts: Opt,
-        default_calls: u8,
-        recursion: u8,
-        default: Option<NonNull<pyo3::ffi::PyObject>>,
-    ) -> Self {
-        DictWithStrKeys {
-            ptr: ptr,
-            opts: opts,
-            default_calls: default_calls,
-            recursion: recursion,
-            default: default,
-        }
-    }
-}
-
-impl Serialize for DictWithStrKeys {
+impl Dict {
     #[inline(always)]
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    fn serialize_with_str_keys<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let len = ffi!(Py_SIZE(self.ptr)) as usize;
+        let len = unsafe { pydict_size(self.ptr) } as usize;
         let mut map = serializer.serialize_map(Some(len)).unwrap();
         for (key, value) in PyDictIter::from_pyobject(self.ptr) {
-            if unlikely!(!py_is!(ob_type!(key.as_ptr()), STR_TYPE)) {
-                err!(KEY_MUST_BE_STR)
-            }
-            let data = unicode_to_str(key.as_ptr());
-            if unlikely!(data.is_none()) {
-                err!(INVALID_STR)
+            if unlikely!(ob_type!(key.as_ptr()) != &raw mut pyo3::ffi::PyUnicode_Type) {
+                return Err(serde::ser::Error::custom(KEY_MUST_BE_STR));
             }
+            let key_as_str = unicode_to_str(key.as_ptr()).map_err(serde::ser::Error::custom)?;
             let pyvalue = PyObject::new(
                 value.as_ptr(),
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion + 1,
                 self.default,
             );
-            map.serialize_key(data.unwrap()).unwrap();
+            map.serialize_key(key_as_str).unwrap();
             map.serialize_value(&pyvalue)?;
         }
         map.end()
     }
-}
 
-pub struct DictWithSortedStrKeys {
-    ptr: *mut pyo3::ffi::PyObject,
-    opts: Opt,
-    default_calls: u8,
-    recursion: u8,
-    default: Option<NonNull<pyo3::ffi::PyObject>>,
-}
-
-impl DictWithSortedStrKeys {
-    pub fn new(
-        ptr: *mut pyo3::ffi::PyObject,
-        opts: Opt,
-        default_calls: u8,
-        recursion: u8,
-        default: Option<NonNull<pyo3::ffi::PyObject>>,
-    ) -> Self {
-        DictWithSortedStrKeys {
-            ptr: ptr,
-            opts: opts,
-            default_calls: default_calls,
-            recursion: recursion,
-            default: default,
-        }
-    }
-}
-
-impl Serialize for DictWithSortedStrKeys {
     #[inline(never)]
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    fn serialize_with_sorted_str_keys<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let len = ffi!(Py_SIZE(self.ptr)) as usize;
+        let len = unsafe { pydict_size(self.ptr) } as usize;
         let mut items: SmallVec<[(&str, *mut pyo3::ffi::PyObject); 8]> =
             SmallVec::with_capacity(len);
         for (key, value) in PyDictIter::from_pyobject(self.ptr) {
-            if unlikely!(!py_is!(ob_type!(key.as_ptr()), STR_TYPE)) {
-                err!(KEY_MUST_BE_STR)
+            if unlikely!(ob_type!(key.as_ptr()) != &raw mut pyo3::ffi::PyUnicode_Type) {
+                return Err(serde::ser::Error::custom(KEY_MUST_BE_STR));
             }
-            let data = unicode_to_str(key.as_ptr());
-            if unlikely!(data.is_none()) {
-                err!(INVALID_STR)
-            }
-            items.push((data.unwrap(), value.as_ptr()));
+            let key_as_str = unicode_to_str(key.as_ptr()).map_err(serde::ser::Error::custom)?;
+            items.push((key_as_str, value.as_ptr()));
         }
 
         items.sort_unstable_by(|a, b| a.0.cmp(b.0));
@@ -186,6 +112,7 @@ impl Serialize for DictWithSortedStrKeys {
         for (key, val) in items.iter() {
             let pyvalue = PyObject::new(
                 *val,
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion + 1,
@@ -196,53 +123,23 @@ impl Serialize for DictWithSortedStrKeys {
         }
         map.end()
     }
-}
 
-pub struct DictWithNonStrKeys {
-    ptr: *mut pyo3::ffi::PyObject,
-    opts: Opt,
-    default_calls: u8,
-    recursion: u8,
-    default: Option<NonNull<pyo3::ffi::PyObject>>,
-}
-
-impl DictWithNonStrKeys {
-    pub fn new(
-        ptr: *mut pyo3::ffi::PyObject,
-        opts: Opt,
-        default_calls: u8,
-        recursion: u8,
-        default: Option<NonNull<pyo3::ffi::PyObject>>,
-    ) -> Self {
-        DictWithNonStrKeys {
-            ptr: ptr,
-            opts: opts,
-            default_calls: default_calls,
-            recursion: recursion,
-            default: default,
-        }
-    }
-}
-
-impl Serialize for DictWithNonStrKeys {
     #[inline(never)]
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    fn serialize_with_non_str_keys<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
         let opts = self.opts & NOT_PASSTHROUGH;
-        let len = ffi!(Py_SIZE(self.ptr)) as usize;
+        let len = unsafe { pydict_size(self.ptr) } as usize;
         let mut map = serializer.serialize_map(Some(len)).unwrap();
         for (key, value) in PyDictIter::from_pyobject(self.ptr) {
-            if py_is!(ob_type!(key.as_ptr()), STR_TYPE) {
-                let data = unicode_to_str(key.as_ptr());
-                if unlikely!(data.is_none()) {
-                    err!(INVALID_STR)
-                }
+            if ob_type!(key.as_ptr()) == &raw mut pyo3::ffi::PyUnicode_Type {
+                let key_as_str = unicode_to_str(key.as_ptr()).map_err(serde::ser::Error::custom)?;
                 map.serialize_entry(
-                    data.unwrap(),
+                    key_as_str,
                     &PyObject::new(
                         value.as_ptr(),
+                        self.state,
                         self.opts,
                         self.default_calls,
                         self.recursion + 1,
@@ -251,9 +148,10 @@ impl Serialize for DictWithNonStrKeys {
                 )?;
             } else {
                 map.serialize_entry(
-                    &DictKey::new(key.as_ptr(), opts, self.recursion + 1),
+                    &DictKey::new(key.as_ptr(), self.state, opts, self.recursion + 1),
                     &PyObject::new(
                         value.as_ptr(),
+                        self.state,
                         self.opts,
                         self.default_calls,
                         self.recursion + 1,
