@@ -1,13 +1,16 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
+use crate::ffi::CriticalSection;
 use crate::opt::*;
 use crate::serialize::serializer::*;
+use crate::state::State;
 
 use serde::ser::{Serialize, SerializeSeq, Serializer};
 use std::ptr::NonNull;
 
 pub struct List {
     ptr: *mut pyo3::ffi::PyObject,
+    state: *mut State,
     opts: Opt,
     default_calls: u8,
     recursion: u8,
@@ -17,6 +20,7 @@ pub struct List {
 impl List {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
+        state: *mut State,
         opts: Opt,
         default_calls: u8,
         recursion: u8,
@@ -24,6 +28,7 @@ impl List {
     ) -> Self {
         List {
             ptr: ptr,
+            state: state,
             opts: opts,
             default_calls: default_calls,
             recursion: recursion,
@@ -37,12 +42,15 @@ impl Serialize for List {
     where
         S: Serializer,
     {
-        let len = ffi!(PyList_GET_SIZE(self.ptr)) as usize;
+        let mut critical_section = CriticalSection::new();
+        critical_section.begin(self.ptr);
+        let len = unsafe { pyo3::ffi::PyList_GET_SIZE(self.ptr) } as usize;
         let mut seq = serializer.serialize_seq(Some(len)).unwrap();
         for i in 0..len {
-            let item = ffi!(PyList_GET_ITEM(self.ptr, i as isize));
+            let item = unsafe { pyo3::ffi::PyList_GET_ITEM(self.ptr, i as isize) };
             let value = PyObject::new(
                 item,
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion + 1,
