@@ -4,22 +4,24 @@ use crate::exc::*;
 use crate::ffi::*;
 use crate::msgpack;
 use crate::opt::*;
+use crate::serialize::bytearray::*;
 use crate::serialize::bytes::*;
 use crate::serialize::dataclass::*;
 use crate::serialize::datetime::*;
 use crate::serialize::default::*;
 use crate::serialize::dict::*;
 use crate::serialize::ext::*;
-use crate::serialize::int::*;
 use crate::serialize::list::*;
+use crate::serialize::memoryview::*;
 use crate::serialize::numpy::*;
 use crate::serialize::pydantic::*;
 use crate::serialize::str::*;
 use crate::serialize::tuple::*;
 use crate::serialize::uuid::*;
 use crate::serialize::writer::*;
-use crate::typeref::*;
+use crate::state::State;
 use serde::ser::{Impossible, Serialize, SerializeMap, SerializeSeq, Serializer};
+use std::os::raw::c_ulong;
 use std::ptr::NonNull;
 
 pub const RECURSION_LIMIT: u8 = 255;
@@ -445,7 +447,10 @@ where
     where
         T: ?Sized + Serialize,
     {
-        let tag: i8 = variant_index.try_into().unwrap_or_else(|_| unreachable!());
+        let tag: i8 = match variant_index {
+            128 => -1,
+            _ => variant_index.try_into().unwrap_or_else(|_| unreachable!()),
+        };
         let mut ext_se = ExtSerializer::new(tag, &mut self.writer);
         value.serialize(&mut ext_se)
     }
@@ -513,40 +518,31 @@ where
 
 pub fn serialize(
     ptr: *mut pyo3::ffi::PyObject,
+    state: *mut State,
     default: Option<NonNull<pyo3::ffi::PyObject>>,
     opts: Opt,
 ) -> Result<NonNull<pyo3::ffi::PyObject>, String> {
     let mut buf = BytesWriter::default();
-    let obj = PyObject::new(ptr, opts, 0, 0, default);
+    let obj = PyObject::new(ptr, state, opts, 0, 0, default);
     let mut ser = MessagePackSerializer::new(&mut buf);
     let res = obj.serialize(&mut ser);
     match res {
         Ok(_) => Ok(buf.finish()),
         Err(err) => {
-            ffi!(Py_DECREF(buf.finish().as_ptr()));
+            unsafe { pyo3::ffi::Py_DECREF(buf.finish().as_ptr()) };
             Err(err.to_string())
         }
     }
 }
 
-macro_rules! is_subclass {
-    ($ob_type:expr, $flag:ident) => {
-        unsafe { (((*$ob_type).tp_flags & pyo3::ffi::$flag) != 0) }
-    };
-}
-
-fn is_big_int(ptr: *mut pyo3::ffi::PyObject) -> bool {
-    ffi!(_PyLong_NumBits(ptr)) > {
-        if pylong_is_positive(ptr) {
-            64
-        } else {
-            63
-        }
-    }
+#[inline(always)]
+fn is_subclass(op: *mut pyo3::ffi::PyTypeObject, feature: c_ulong) -> bool {
+    unsafe { pyo3::ffi::PyType_HasFeature(op, feature) != 0 }
 }
 
 pub struct PyObject {
     ptr: *mut pyo3::ffi::PyObject,
+    state: *mut State,
     opts: Opt,
     default_calls: u8,
     recursion: u8,
@@ -556,6 +552,7 @@ pub struct PyObject {
 impl PyObject {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
+        state: *mut State,
         opts: Opt,
         default_calls: u8,
         recursion: u8,
@@ -563,6 +560,7 @@ impl PyObject {
     ) -> Self {
         PyObject {
             ptr: ptr,
+            state: state,
             opts: opts,
             default_calls: default_calls,
             recursion: recursion,
@@ -578,29 +576,31 @@ impl PyObject {
         let ob_type = ob_type!(self.ptr);
 
         if self.opts & PASSTHROUGH_DATETIME == 0 {
-            if py_is!(ob_type, DATETIME_TYPE) {
-                match DateTime::new(self.ptr, self.opts) {
+            let datetime_api = unsafe { *pyo3::ffi::PyDateTimeAPI() };
+            if ob_type == datetime_api.DateTimeType {
+                match DateTime::new(self.ptr, self.state, self.opts) {
                     Ok(val) => return val.serialize(serializer),
-                    Err(err) => err!(err),
+                    Err(err) => return Err(serde::ser::Error::custom(err)),
                 }
             }
-            if py_is!(ob_type, DATE_TYPE) {
+            if ob_type == datetime_api.DateType {
                 return Date::new(self.ptr).serialize(serializer);
             }
-            if py_is!(ob_type, TIME_TYPE) {
+            if ob_type == datetime_api.TimeType {
                 match Time::new(self.ptr, self.opts) {
                     Ok(val) => return val.serialize(serializer),
-                    Err(err) => err!(err),
+                    Err(err) => return Err(serde::ser::Error::custom(err)),
                 };
             }
         }
 
-        if self.opts & PASSTHROUGH_TUPLE == 0 && py_is!(ob_type, TUPLE_TYPE) {
+        if self.opts & PASSTHROUGH_TUPLE == 0 && ob_type == &raw mut pyo3::ffi::PyTuple_Type {
             if unlikely!(self.recursion == RECURSION_LIMIT) {
-                err!(RECURSION_LIMIT_REACHED)
+                return Err(serde::ser::Error::custom(RECURSION_LIMIT_REACHED));
             }
             return Tuple::new(
                 self.ptr,
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion,
@@ -609,16 +609,18 @@ impl PyObject {
             .serialize(serializer);
         }
 
-        if self.opts & PASSTHROUGH_UUID == 0 && py_is!(ob_type, UUID_TYPE) {
-            return UUID::new(self.ptr).serialize(serializer);
+        if self.opts & PASSTHROUGH_UUID == 0 && ob_type == unsafe { (*self.state).uuid_type } {
+            return UUID::new(self.ptr, self.state).serialize(serializer);
         }
 
-        if py_is!(ob_type!(ob_type), ENUM_TYPE) {
+        if ob_type!(ob_type) == unsafe { (*self.state).enum_type } {
             if self.opts & PASSTHROUGH_ENUM == 0 {
-                let value = ffi!(PyObject_GetAttr(self.ptr, VALUE_STR));
-                ffi!(Py_DECREF(value));
+                let value =
+                    unsafe { pyo3::ffi::PyObject_GetAttr(self.ptr, (*self.state).value_str) };
+                unsafe { pyo3::ffi::Py_DECREF(value) };
                 return PyObject::new(
                     value,
+                    self.state,
                     self.opts,
                     self.default_calls,
                     self.recursion,
@@ -628,6 +630,7 @@ impl PyObject {
             } else {
                 return Default::new(
                     self.ptr,
+                    self.state,
                     self.opts,
                     self.default_calls,
                     self.recursion,
@@ -638,20 +641,36 @@ impl PyObject {
         }
 
         if self.opts & PASSTHROUGH_SUBCLASS == 0 {
-            if is_subclass!(ob_type, Py_TPFLAGS_UNICODE_SUBCLASS) {
-                return StrSubclass::new(self.ptr).serialize(serializer);
+            if is_subclass(ob_type, pyo3::ffi::Py_TPFLAGS_UNICODE_SUBCLASS) {
+                return StrSubclass::new(self.ptr, self.opts).serialize(serializer);
             }
-            if is_subclass!(ob_type, Py_TPFLAGS_LONG_SUBCLASS)
-                && (self.opts & PASSTHROUGH_BIG_INT == 0 || !is_big_int(self.ptr))
-            {
-                return Int::new(self.ptr).serialize(serializer);
+            if is_subclass(ob_type, pyo3::ffi::Py_TPFLAGS_LONG_SUBCLASS) {
+                match Int::new(self.ptr) {
+                    Ok(val) => return val.serialize(serializer),
+                    Err(err) => {
+                        if self.opts & PASSTHROUGH_BIG_INT != 0 {
+                            return Default::new(
+                                self.ptr,
+                                self.state,
+                                self.opts,
+                                self.default_calls,
+                                self.recursion,
+                                self.default,
+                            )
+                            .serialize(serializer);
+                        } else {
+                            return Err(serde::ser::Error::custom(err));
+                        }
+                    }
+                }
             }
-            if is_subclass!(ob_type, Py_TPFLAGS_LIST_SUBCLASS) {
+            if is_subclass(ob_type, pyo3::ffi::Py_TPFLAGS_LIST_SUBCLASS) {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
-                    err!(RECURSION_LIMIT_REACHED)
+                    return Err(serde::ser::Error::custom(RECURSION_LIMIT_REACHED));
                 }
                 return List::new(
                     self.ptr,
+                    self.state,
                     self.opts,
                     self.default_calls,
                     self.recursion,
@@ -659,12 +678,13 @@ impl PyObject {
                 )
                 .serialize(serializer);
             }
-            if is_subclass!(ob_type, Py_TPFLAGS_DICT_SUBCLASS) {
+            if is_subclass(ob_type, pyo3::ffi::Py_TPFLAGS_DICT_SUBCLASS) {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
-                    err!(RECURSION_LIMIT_REACHED)
+                    return Err(serde::ser::Error::custom(RECURSION_LIMIT_REACHED));
                 }
                 return Dict::new(
                     self.ptr,
+                    self.state,
                     self.opts,
                     self.default_calls,
                     self.recursion,
@@ -674,17 +694,17 @@ impl PyObject {
             }
         }
 
-        if py_is!(ob_type, EXT_TYPE) {
+        if ob_type == unsafe { (*self.state).ext_type } {
             return Ext::new(self.ptr).serialize(serializer);
         }
 
-        if self.opts & PASSTHROUGH_DATACLASS == 0 && pydict_contains!(ob_type, DATACLASS_FIELDS_STR)
-        {
+        if self.opts & PASSTHROUGH_DATACLASS == 0 && is_dataclass(ob_type, self.state) {
             if unlikely!(self.recursion == RECURSION_LIMIT) {
-                err!(RECURSION_LIMIT_REACHED)
+                return Err(serde::ser::Error::custom(RECURSION_LIMIT_REACHED));
             }
             return Dataclass::new(
                 self.ptr,
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion,
@@ -693,33 +713,31 @@ impl PyObject {
             .serialize(serializer);
         }
 
-        if self.opts & SERIALIZE_PYDANTIC != 0
-            && (pydict_contains!(ob_type, PYDANTIC_FIELDS_STR)
-                || pydict_contains!(ob_type, PYDANTIC2_VALIDATOR_STR))
-        {
+        if self.opts & SERIALIZE_PYDANTIC != 0 && is_pydantic_model(ob_type, self.state) {
             if unlikely!(self.recursion == RECURSION_LIMIT) {
-                err!(RECURSION_LIMIT_REACHED)
+                return Err(serde::ser::Error::custom(RECURSION_LIMIT_REACHED));
             }
             match PydanticModel::new(
                 self.ptr,
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion,
                 self.default,
             ) {
                 Ok(val) => return val.serialize(serializer),
-                Err(err) => err!(err),
+                Err(err) => return Err(serde::ser::Error::custom(err)),
             };
         }
 
         if self.opts & SERIALIZE_NUMPY != 0 {
-            if let Some(numpy_types) = unsafe { NUMPY_TYPES.get_or_init(load_numpy_types) } {
-                let numpy_types_ref = unsafe { numpy_types.as_ref() };
+            if let Some(numpy_types_ref) = unsafe { (*self.state).get_numpy_types() } {
                 if ob_type == numpy_types_ref.bool_ {
                     return NumpyBool::new(self.ptr).serialize(serializer);
                 }
                 if ob_type == numpy_types_ref.datetime64 {
-                    return NumpyDatetime64::new(self.ptr, self.opts).serialize(serializer);
+                    return NumpyDatetime64::new(self.ptr, self.state, self.opts)
+                        .serialize(serializer);
                 }
                 if ob_type == numpy_types_ref.float16 {
                     return NumpyFloat16::new(self.ptr).serialize(serializer);
@@ -755,13 +773,15 @@ impl PyObject {
                     return NumpyUint64::new(self.ptr).serialize(serializer);
                 }
                 if ob_type == numpy_types_ref.array {
-                    match NumpyArray::new(self.ptr, self.opts) {
+                    match NumpyArray::new(self.ptr, self.state, self.opts) {
                         Ok(val) => return val.serialize(serializer),
-                        Err(PyArrayError::Malformed) => err!("numpy array is malformed"),
+                        Err(PyArrayError::Malformed) => {
+                            return Err(serde::ser::Error::custom("numpy array is malformed"))
+                        }
                         Err(PyArrayError::NotContiguous)
                         | Err(PyArrayError::UnsupportedDataType) => {
                             if self.default.is_none() {
-                                err!("numpy array is not C contiguous; use ndarray.tolist() in default")
+                                return Err(serde::ser::Error::custom("numpy array is not C contiguous; use ndarray.tolist() in default"));
                             }
                         }
                     }
@@ -769,8 +789,16 @@ impl PyObject {
             }
         }
 
+        if ob_type == &raw mut pyo3::ffi::PyByteArray_Type {
+            return ByteArray::new(self.ptr).serialize(serializer);
+        }
+        if ob_type == &raw mut pyo3::ffi::PyMemoryView_Type {
+            return MemoryView::new(self.ptr).serialize(serializer);
+        }
+
         Default::new(
             self.ptr,
+            self.state,
             self.opts,
             self.default_calls,
             self.recursion,
@@ -786,38 +814,55 @@ impl Serialize for PyObject {
         S: Serializer,
     {
         let ob_type = ob_type!(self.ptr);
-        if py_is!(ob_type, STR_TYPE) {
-            Str::new(self.ptr).serialize(serializer)
-        } else if py_is!(ob_type, BYTES_TYPE) {
+        if ob_type == &raw mut pyo3::ffi::PyUnicode_Type {
+            Str::new(self.ptr, self.opts).serialize(serializer)
+        } else if ob_type == &raw mut pyo3::ffi::PyBytes_Type {
             Bytes::new(self.ptr).serialize(serializer)
-        } else if py_is!(ob_type, INT_TYPE)
-            && (self.opts & PASSTHROUGH_BIG_INT == 0 || !is_big_int(self.ptr))
-        {
-            Int::new(self.ptr).serialize(serializer)
-        } else if py_is!(ob_type, BOOL_TYPE) {
-            serializer.serialize_bool(unsafe { self.ptr == TRUE })
-        } else if py_is!(self.ptr, NONE) {
+        } else if ob_type == &raw mut pyo3::ffi::PyLong_Type {
+            match Int::new(self.ptr) {
+                Ok(val) => val.serialize(serializer),
+                Err(err) => {
+                    if self.opts & PASSTHROUGH_BIG_INT != 0 {
+                        Default::new(
+                            self.ptr,
+                            self.state,
+                            self.opts,
+                            self.default_calls,
+                            self.recursion,
+                            self.default,
+                        )
+                        .serialize(serializer)
+                    } else {
+                        Err(serde::ser::Error::custom(err))
+                    }
+                }
+            }
+        } else if ob_type == &raw mut pyo3::ffi::PyBool_Type {
+            serializer.serialize_bool(unsafe { self.ptr == pyo3::ffi::Py_True() })
+        } else if self.ptr == unsafe { pyo3::ffi::Py_None() } {
             serializer.serialize_unit()
-        } else if py_is!(ob_type, FLOAT_TYPE) {
-            serializer.serialize_f64(ffi!(PyFloat_AS_DOUBLE(self.ptr)))
-        } else if py_is!(ob_type, LIST_TYPE) {
+        } else if ob_type == &raw mut pyo3::ffi::PyFloat_Type {
+            serializer.serialize_f64(unsafe { pyo3::ffi::PyFloat_AS_DOUBLE(self.ptr) })
+        } else if ob_type == &raw mut pyo3::ffi::PyList_Type {
             if unlikely!(self.recursion == RECURSION_LIMIT) {
-                err!(RECURSION_LIMIT_REACHED)
+                return Err(serde::ser::Error::custom(RECURSION_LIMIT_REACHED));
             }
             List::new(
                 self.ptr,
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion,
                 self.default,
             )
             .serialize(serializer)
-        } else if py_is!(ob_type, DICT_TYPE) {
+        } else if ob_type == &raw mut pyo3::ffi::PyDict_Type {
             if unlikely!(self.recursion == RECURSION_LIMIT) {
-                err!(RECURSION_LIMIT_REACHED)
+                return Err(serde::ser::Error::custom(RECURSION_LIMIT_REACHED));
             }
             Dict::new(
                 self.ptr,
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion,
@@ -832,14 +877,16 @@ impl Serialize for PyObject {
 
 pub struct DictTupleKey {
     ptr: *mut pyo3::ffi::PyObject,
+    state: *mut State,
     opts: Opt,
     recursion: u8,
 }
 
 impl DictTupleKey {
-    pub fn new(ptr: *mut pyo3::ffi::PyObject, opts: Opt, recursion: u8) -> Self {
+    pub fn new(ptr: *mut pyo3::ffi::PyObject, state: *mut State, opts: Opt, recursion: u8) -> Self {
         DictTupleKey {
             ptr: ptr,
+            state: state,
             opts: opts,
             recursion: recursion,
         }
@@ -852,11 +899,11 @@ impl Serialize for DictTupleKey {
     where
         S: Serializer,
     {
-        let len = ffi!(PyTuple_GET_SIZE(self.ptr)) as usize;
+        let len = unsafe { pyo3::ffi::Py_SIZE(self.ptr) } as usize;
         let mut seq = serializer.serialize_seq(Some(len)).unwrap();
         for i in 0..len {
-            let item = ffi!(PyTuple_GET_ITEM(self.ptr, i as isize));
-            let value = DictKey::new(item, self.opts, self.recursion + 1);
+            let item = unsafe { pytuple_get_item(self.ptr, i as isize) };
+            let value = DictKey::new(item, self.state, self.opts, self.recursion + 1);
             seq.serialize_element(&value)?;
         }
         seq.end()
@@ -865,14 +912,16 @@ impl Serialize for DictTupleKey {
 
 pub struct DictKey {
     ptr: *mut pyo3::ffi::PyObject,
+    state: *mut State,
     opts: Opt,
     recursion: u8,
 }
 
 impl DictKey {
-    pub fn new(ptr: *mut pyo3::ffi::PyObject, opts: Opt, recursion: u8) -> Self {
+    pub fn new(ptr: *mut pyo3::ffi::PyObject, state: *mut State, opts: Opt, recursion: u8) -> Self {
         DictKey {
             ptr: ptr,
+            state: state,
             opts: opts,
             recursion: recursion,
         }
@@ -885,51 +934,59 @@ impl DictKey {
     {
         let ob_type = ob_type!(self.ptr);
 
-        if py_is!(ob_type, DATETIME_TYPE) {
-            match DateTime::new(self.ptr, self.opts) {
+        let datetime_api = unsafe { *pyo3::ffi::PyDateTimeAPI() };
+        if ob_type == datetime_api.DateTimeType {
+            match DateTime::new(self.ptr, self.state, self.opts) {
                 Ok(val) => return val.serialize(serializer),
-                Err(err) => err!(err),
+                Err(err) => return Err(serde::ser::Error::custom(err)),
             }
         }
-        if py_is!(ob_type, DATE_TYPE) {
+        if ob_type == datetime_api.DateType {
             return Date::new(self.ptr).serialize(serializer);
         }
-        if py_is!(ob_type, TIME_TYPE) {
+        if ob_type == datetime_api.TimeType {
             match Time::new(self.ptr, self.opts) {
                 Ok(val) => return val.serialize(serializer),
-                Err(err) => err!(err),
+                Err(err) => return Err(serde::ser::Error::custom(err)),
             };
         }
 
-        if py_is!(ob_type, TUPLE_TYPE) {
+        if ob_type == &raw mut pyo3::ffi::PyTuple_Type {
             if unlikely!(self.recursion == RECURSION_LIMIT) {
-                err!(RECURSION_LIMIT_REACHED)
+                return Err(serde::ser::Error::custom(RECURSION_LIMIT_REACHED));
             }
-            return DictTupleKey::new(self.ptr, self.opts, self.recursion).serialize(serializer);
+            return DictTupleKey::new(self.ptr, self.state, self.opts, self.recursion)
+                .serialize(serializer);
         }
 
-        if py_is!(ob_type, UUID_TYPE) {
-            return UUID::new(self.ptr).serialize(serializer);
+        if ob_type == unsafe { (*self.state).uuid_type } {
+            return UUID::new(self.ptr, self.state).serialize(serializer);
         }
 
-        if py_is!(ob_type!(ob_type), ENUM_TYPE) {
-            let value = ffi!(PyObject_GetAttr(self.ptr, VALUE_STR));
-            ffi!(Py_DECREF(value));
-            return DictKey::new(value, self.opts, self.recursion).serialize(serializer);
+        if ob_type!(ob_type) == unsafe { (*self.state).enum_type } {
+            let value = unsafe { pyo3::ffi::PyObject_GetAttr(self.ptr, (*self.state).value_str) };
+            unsafe { pyo3::ffi::Py_DECREF(value) };
+            return DictKey::new(value, self.state, self.opts, self.recursion)
+                .serialize(serializer);
         }
 
-        if is_subclass!(ob_type, Py_TPFLAGS_UNICODE_SUBCLASS) {
-            return StrSubclass::new(self.ptr).serialize(serializer);
+        if is_subclass(ob_type, pyo3::ffi::Py_TPFLAGS_UNICODE_SUBCLASS) {
+            return StrSubclass::new(self.ptr, self.opts).serialize(serializer);
         }
-        if is_subclass!(ob_type, Py_TPFLAGS_LONG_SUBCLASS) {
-            return Int::new(self.ptr).serialize(serializer);
+        if is_subclass(ob_type, pyo3::ffi::Py_TPFLAGS_LONG_SUBCLASS) {
+            match Int::new(self.ptr) {
+                Ok(val) => return val.serialize(serializer),
+                Err(err) => return Err(serde::ser::Error::custom(err)),
+            }
         }
 
-        if py_is!(ob_type, EXT_TYPE) {
-            return Ext::new(self.ptr).serialize(serializer);
+        if ob_type == &raw mut pyo3::ffi::PyMemoryView_Type {
+            return MemoryView::new(self.ptr).serialize(serializer);
         }
 
-        err!("Dict key must a type serializable with OPT_NON_STR_KEYS")
+        Err(serde::ser::Error::custom(
+            "Dict key must a type serializable with OPT_NON_STR_KEYS",
+        ))
     }
 }
 
@@ -939,18 +996,21 @@ impl Serialize for DictKey {
         S: Serializer,
     {
         let ob_type = ob_type!(self.ptr);
-        if py_is!(ob_type, STR_TYPE) {
-            Str::new(self.ptr).serialize(serializer)
-        } else if py_is!(ob_type, BYTES_TYPE) {
+        if ob_type == &raw mut pyo3::ffi::PyUnicode_Type {
+            Str::new(self.ptr, self.opts).serialize(serializer)
+        } else if ob_type == &raw mut pyo3::ffi::PyBytes_Type {
             Bytes::new(self.ptr).serialize(serializer)
-        } else if py_is!(ob_type, INT_TYPE) {
-            Int::new(self.ptr).serialize(serializer)
-        } else if py_is!(ob_type, BOOL_TYPE) {
-            serializer.serialize_bool(unsafe { self.ptr == TRUE })
-        } else if py_is!(self.ptr, NONE) {
+        } else if ob_type == &raw mut pyo3::ffi::PyLong_Type {
+            match Int::new(self.ptr) {
+                Ok(val) => val.serialize(serializer),
+                Err(err) => Err(serde::ser::Error::custom(err)),
+            }
+        } else if ob_type == &raw mut pyo3::ffi::PyBool_Type {
+            serializer.serialize_bool(unsafe { self.ptr == pyo3::ffi::Py_True() })
+        } else if self.ptr == unsafe { pyo3::ffi::Py_None() } {
             serializer.serialize_unit()
-        } else if py_is!(ob_type, FLOAT_TYPE) {
-            serializer.serialize_f64(ffi!(PyFloat_AS_DOUBLE(self.ptr)))
+        } else if ob_type == &raw mut pyo3::ffi::PyFloat_Type {
+            serializer.serialize_f64(unsafe { pyo3::ffi::PyFloat_AS_DOUBLE(self.ptr) })
         } else {
             self.serialize_unlikely(serializer)
         }
