@@ -1,13 +1,16 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
+use crate::ffi::*;
 use crate::opt::*;
 use crate::serialize::serializer::*;
+use crate::state::State;
 
 use serde::ser::{Serialize, SerializeSeq, Serializer};
 use std::ptr::NonNull;
 
 pub struct Tuple {
     ptr: *mut pyo3::ffi::PyObject,
+    state: *mut State,
     opts: Opt,
     default_calls: u8,
     recursion: u8,
@@ -17,6 +20,7 @@ pub struct Tuple {
 impl Tuple {
     pub fn new(
         ptr: *mut pyo3::ffi::PyObject,
+        state: *mut State,
         opts: Opt,
         default_calls: u8,
         recursion: u8,
@@ -24,6 +28,7 @@ impl Tuple {
     ) -> Self {
         Tuple {
             ptr: ptr,
+            state: state,
             opts: opts,
             default_calls: default_calls,
             recursion: recursion,
@@ -38,12 +43,13 @@ impl Serialize for Tuple {
     where
         S: Serializer,
     {
-        let len = ffi!(PyTuple_GET_SIZE(self.ptr)) as usize;
+        let len = unsafe { pyo3::ffi::Py_SIZE(self.ptr) } as usize;
         let mut seq = serializer.serialize_seq(Some(len)).unwrap();
         for i in 0..len {
-            let item = ffi!(PyTuple_GET_ITEM(self.ptr, i as isize));
+            let item = unsafe { pytuple_get_item(self.ptr, i as isize) };
             let value = PyObject::new(
                 item,
+                self.state,
                 self.opts,
                 self.default_calls,
                 self.recursion + 1,
